@@ -1,6 +1,6 @@
 package YaraFFI;
 
-$YaraFFI::VERSION   = '0.04';
+$YaraFFI::VERSION   = '0.05';
 $YaraFFI::AUTHORITY = 'cpan:MANWAR';
 
 =head1 NAME
@@ -9,7 +9,7 @@ YaraFFI - Minimal Perl FFI bindings for the YARA malware scanning engine
 
 =head1 VERSION
 
-Version 0.04
+Version 0.05
 
 =head1 SYNOPSIS
 
@@ -99,6 +99,10 @@ use v5.14;
 use strict;
 use warnings;
 use YaraFFI::Event;
+use YaraFFI::Record::YR_RULE;
+use YaraFFI::Record::YR_META;
+use YaraFFI::Record::YR_STRING;
+use YaraFFI::Record::YR_MATCH;
 use FFI::Platypus 2.00;
 use File::Slurp qw(read_file);
 
@@ -164,121 +168,125 @@ sub scan_file {
     return $self->scan_buffer($content, $callback);
 }
 
-# Safe memory reading helper with bounds checking
-sub _safe_read_ptr {
-    my ($addr) = @_;
-    use bigint;
-    my $max_addr = 140737488355327;  # 0x7fffffffffff
-    no bigint;
-
-    return 0 unless $addr && $addr > 4096 && $addr < $max_addr;
-
-    my $ptr_bytes = eval { $ffi->cast('opaque' => 'string(8)', $addr) };
-    return 0 if $@ || !defined $ptr_bytes || length($ptr_bytes) < 8;
-
-    my $value = unpack('Q', $ptr_bytes);
-    return ($value > 4096 && $value < $max_addr) ? $value : 0;
-}
-
-sub _safe_read_string {
-    my ($addr) = @_;
-    use bigint;
-    my $max_addr = 140737488355327;  # 0x7fffffffffff
-    no bigint;
-
-    return undef unless $addr && $addr > 4096 && $addr < $max_addr;
-
-    my $str = eval { $ffi->cast('opaque' => 'string', $addr) };
-    return undef if $@;
-
-    # Basic sanity check - string should be printable-ish
-    return undef if !defined $str || length($str) > 1000;
-    return $str;
-}
-
-# Safe metadata extraction with extensive error checking
-sub _extract_metadata {
+# Safe metadata extraction using FFI records
+sub _extract_metadata_with_records {
     my ($rule_ptr, $enable_extraction) = @_;
     return undef unless $enable_extraction;
     return undef unless $rule_ptr && $rule_ptr > 4096;
 
     my %metadata;
 
-    # Search for metadata pointer in likely offsets
-    for my $meta_offset (16, 24, 32, 40, 48) {
-        my $meta_ptr = _safe_read_ptr($rule_ptr + $meta_offset);
-        next unless $meta_ptr;
+    eval {
+        # Cast to YR_RULE record
+        my $rule = $ffi->cast('opaque', 'record(YaraFFI::Record::YR_RULE)*', $rule_ptr);
+        return unless $rule && $rule->metas;
 
-        # Try to read a few metadata entries
-        for my $i (0..9) {
-            my $entry_base = $meta_ptr + ($i * 32);
+        my $metas_ptr = $rule->metas;
+        return unless $metas_ptr > 4096;
 
-            # Read type carefully
-            my $type_bytes = eval { $ffi->cast('opaque' => 'string(4)', $entry_base) };
+        # Iterate through metadata entries
+        for my $i (0..19) {
+            my $meta_ptr = $metas_ptr + ($i * 32);  # Approximate YR_META size
+
+            my $type_bytes = $ffi->cast('opaque', 'string(4)', $meta_ptr);
             last unless defined $type_bytes && length($type_bytes) == 4;
 
             my $type = unpack('L', $type_bytes);
             last if $type == 0 || $type > 3;
 
-            my $id_ptr = _safe_read_ptr($entry_base + 8);
-            next unless $id_ptr;
+            # Read identifier pointer
+            my $id_ptr_bytes = $ffi->cast('opaque', 'string(8)', $meta_ptr + 8);
+            next unless defined $id_ptr_bytes && length($id_ptr_bytes) == 8;
 
-            my $identifier = _safe_read_string($id_ptr);
+            my $id_ptr = unpack('Q', $id_ptr_bytes);
+            next unless $id_ptr > 4096;
+
+            my $identifier = $ffi->cast('opaque', 'string', $id_ptr);
             next unless $identifier && $identifier =~ /^[a-zA-Z_][a-zA-Z0-9_]{0,100}$/;
 
             if ($type == META_TYPE_STRING) {
-                my $str_ptr = _safe_read_ptr($entry_base + 16);
-                if ($str_ptr) {
-                    my $value = _safe_read_string($str_ptr);
-                    $metadata{$identifier} = $value if defined $value;
+                my $str_ptr_bytes = $ffi->cast('opaque', 'string(8)', $meta_ptr + 16);
+                if (defined $str_ptr_bytes && length($str_ptr_bytes) == 8) {
+                    my $str_ptr = unpack('Q', $str_ptr_bytes);
+                    if ($str_ptr > 4096) {
+                        my $value = $ffi->cast('opaque', 'string', $str_ptr);
+                        $metadata{$identifier} = $value if defined $value && length($value) < 1000;
+                    }
+                }
+            }
+            elsif ($type == META_TYPE_INTEGER) {
+                my $int_bytes = $ffi->cast('opaque', 'string(8)', $meta_ptr + 16);
+                if (defined $int_bytes && length($int_bytes) == 8) {
+                    my $value = unpack('q', $int_bytes);  # signed 64-bit
+                    $metadata{$identifier} = $value;
+                }
+            }
+            elsif ($type == META_TYPE_BOOLEAN) {
+                my $bool_bytes = $ffi->cast('opaque', 'string(4)', $meta_ptr + 16);
+                if (defined $bool_bytes && length($bool_bytes) == 4) {
+                    my $value = unpack('L', $bool_bytes);
+                    $metadata{$identifier} = $value ? 1 : 0;
                 }
             }
         }
-
-        last if keys %metadata > 0;
-    }
+    };
 
     return keys %metadata > 0 ? \%metadata : undef;
 }
 
-# Safe string match extraction with extensive error checking
-sub _extract_string_matches {
+# Safe string match extraction using FFI records
+sub _extract_string_matches_with_records {
     my ($rule_ptr, $enable_extraction) = @_;
     return [] unless $enable_extraction;
     return [] unless $rule_ptr && $rule_ptr > 4096;
 
     my @strings;
 
-    # Search for strings pointer in likely offsets
-    for my $str_offset (64, 72, 80, 88, 96) {
-        my $strings_ptr = _safe_read_ptr($rule_ptr + $str_offset);
-        next unless $strings_ptr;
+    eval {
+        # Cast to YR_RULE record
+        my $rule = $ffi->cast('opaque', 'record(YaraFFI::Record::YR_RULE)*', $rule_ptr);
+        return unless $rule && $rule->strings;
 
-        # Try to read string entries
-        for my $i (0..19) {
-            my $string_base = $strings_ptr + ($i * 64);
+        my $strings_ptr = $rule->strings;
+        return unless $strings_ptr > 4096;
 
-            my $id_ptr = _safe_read_ptr($string_base);
-            last unless $id_ptr;
+        # Iterate through string entries
+        for my $i (0..49) {
+            my $string_base = $strings_ptr + ($i * 64);  # Approximate YR_STRING size
 
-            my $identifier = _safe_read_string($id_ptr);
+            # Read identifier pointer
+            my $id_ptr_bytes = $ffi->cast('opaque', 'string(8)', $string_base);
+            last unless defined $id_ptr_bytes && length($id_ptr_bytes) == 8;
+
+            my $id_ptr = unpack('Q', $id_ptr_bytes);
+            last unless $id_ptr > 4096;
+
+            my $identifier = $ffi->cast('opaque', 'string', $id_ptr);
             last unless $identifier && $identifier =~ /^\$[a-zA-Z_][a-zA-Z0-9_]{0,100}$/;
 
-            # Try to find matches
+            # Try to find matches for this string
             my @offsets;
             for my $match_offset (40, 48, 56) {
-                my $matches_ptr = _safe_read_ptr($string_base + $match_offset);
-                next unless $matches_ptr;
+                my $matches_ptr_bytes = $ffi->cast('opaque', 'string(8)', $string_base + $match_offset);
+                next unless defined $matches_ptr_bytes && length($matches_ptr_bytes) == 8;
 
-                for my $j (0..99) {
-                    my $match_base = $matches_ptr + ($j * 24);
+                my $matches_ptr = unpack('Q', $matches_ptr_bytes);
+                next unless $matches_ptr > 4096;
 
-                    my $offset_bytes = eval { $ffi->cast('opaque' => 'string(8)', $match_base) };
+                # Read match structures
+                for my $j (0..199) {
+                    my $match_base = $matches_ptr + ($j * 24);  # YR_MATCH size
+
+                    my $offset_bytes = $ffi->cast('opaque', 'string(8)', $match_base + 8);
                     last unless defined $offset_bytes && length($offset_bytes) == 8;
 
                     my $offset = unpack('Q', $offset_bytes);
                     last if $offset == 0 && $j > 0;
-                    last if $offset > 0xFFFFFFFF;
+
+                    use bigint;
+                    my $max_offset = 4294967295;
+                    no bigint;
+                    last if $offset > $max_offset;
 
                     push @offsets, $offset;
                 }
@@ -291,9 +299,7 @@ sub _extract_string_matches {
                 offsets => \@offsets,
             } if @offsets > 0;
         }
-
-        last if @strings > 0;
-    }
+    };
 
     return \@strings;
 }
@@ -323,21 +329,24 @@ sub scan_buffer {
                     my $rule_name;
                     eval {
                         my $identifier_field_addr = $message_data + $offset;
-                        my $ptr_bytes = $ffi->cast('opaque' => 'string(8)', $identifier_field_addr);
+                        my $ptr_bytes = $ffi->cast('opaque', 'string(8)', $identifier_field_addr);
                         my $name_ptr_value = unpack('Q', $ptr_bytes);
 
                         return if $name_ptr_value < 4096;
-                        return if $name_ptr_value > 140737488355327;
+                        use bigint;
+                        my $max_addr = 140737488355327;  # 0x7fffffffffff
+                        no bigint;
+                        return if $name_ptr_value > $max_addr;
 
-                        $rule_name = $ffi->cast('opaque' => 'string', $name_ptr_value);
+                        $rule_name = $ffi->cast('opaque', 'string', $name_ptr_value);
                     };
 
                     if (!$@ && defined $rule_name && $rule_name =~ /^[a-zA-Z_][a-zA-Z0-9_]*$/) {
 
-                        # Extract metadata only if enabled
+                        # Extract metadata only if enabled (try record-based approach first)
                         my $metadata;
                         if ($enable_metadata) {
-                            $metadata = eval { _extract_metadata($message_data, 1) };
+                            $metadata = eval { _extract_metadata_with_records($message_data, 1) };
                             $metadata = undef if $@;
                         }
 
@@ -358,9 +367,9 @@ sub scan_buffer {
                         if ($emit_strings) {
                             my @string_events;
 
-                            # Try to extract detailed string matches if enabled
+                            # Try to extract detailed string matches if enabled (record-based)
                             if ($enable_offsets) {
-                                my $strings = eval { _extract_string_matches($message_data, 1) };
+                                my $strings = eval { _extract_string_matches_with_records($message_data, 1) };
 
                                 if (!$@ && $strings && ref $strings eq 'ARRAY' && @$strings) {
                                     for my $string (@$strings) {
@@ -469,6 +478,23 @@ due to YARA version compatibility issues. Enable them at your own risk:
         enable_offsets => 1
     );
 
+These features use FFI::Platypus::Record to safely access YARA internal structures,
+but structure layouts may vary between YARA versions.
+
+=head1 DEPENDENCIES
+
+=over 4
+
+=item * FFI::Platypus 2.00 or higher
+
+=item * FFI::Platypus::Record (for metadata/offset features)
+
+=item * File::Slurp
+
+=item * libyara.so (YARA library)
+
+=back
+
 =head1 AUTHOR
 
 Mohammad Sajid Anwar, C<< <mohammad.anwar at yahoo.com> >>
